/*************************************************************************/ /*!
@File
@Title          Wrapper of libnativewindow
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@License        MIT

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/ /**************************************************************************/

#ifndef IMG_NATIVEWINDOW_H
#define IMG_NATIVEWINDOW_H

/* The header includes system/window.h or vndk/window.h from Android
 * nativewindow. It wraps nativewindow APIs with compatibility according to
 * API_LEVEL. It is required linking to libnativewindow with this header.
 * Header system/window.h is superset of vndk/window.h
 * Returns 0 on success or -errno on error.
 * -ENOSYS on functions not supported in specified API_LEVEL.
 */

#if (API_LEVEL >= 31)
#include <vndk/window.h>
#ifndef ANDROID_NATIVE_WINDOW_MAGIC
#define ANDROID_NATIVE_WINDOW_MAGIC ANDROID_NATIVE_MAKE_CONSTANT('_','w','n','d')
#endif
#else
#include <system/window.h>
#endif

#define IMG_METADATA_MAGIC    ANDROID_NATIVE_MAKE_CONSTANT('m','e','t','a')
#define IMG_RESERVED_MAGIC    ANDROID_NATIVE_MAKE_CONSTANT('r','e','s','v')

static inline int
AndroidNativeWindow_queryBufferAge(const ANativeWindow *psNativeWindow,
                                   int *piValue)
{
#if (API_LEVEL >= 31)
	return ANativeWindow_query(psNativeWindow, ANATIVEWINDOW_QUERY_BUFFER_AGE,
	                           piValue);
#else
	return psNativeWindow->query(psNativeWindow, NATIVE_WINDOW_BUFFER_AGE,
	                             piValue);
#endif
}

static inline int
AndroidNativeWindow_queryDefaultHeight(const ANativeWindow *psNativeWindow,
                                       int* piValue)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_query(psNativeWindow,
	                           ANATIVEWINDOW_QUERY_DEFAULT_HEIGHT, piValue);
#else
	return psNativeWindow->query(psNativeWindow,
	                             NATIVE_WINDOW_DEFAULT_HEIGHT, piValue);
#endif
}

static inline int
AndroidNativeWindow_queryDefaultWidth(const ANativeWindow *psNativeWindow,
                                      int *piValue)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_query(psNativeWindow,
	                           ANATIVEWINDOW_QUERY_DEFAULT_WIDTH, piValue);
#else
	return psNativeWindow->query(psNativeWindow,
	                             NATIVE_WINDOW_DEFAULT_WIDTH, piValue);
#endif
}

static inline int
AndroidNativeWindow_queryMinUndequeuedBuffers(
                                            const ANativeWindow *psNativeWindow,
                                            int *piValue)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_query(psNativeWindow,
	                           ANATIVEWINDOW_QUERY_MIN_UNDEQUEUED_BUFFERS,
	                           piValue);
#else
	return psNativeWindow->query(psNativeWindow,
	                             NATIVE_WINDOW_MIN_UNDEQUEUED_BUFFERS, piValue);
#endif
}

static inline int
AndroidNativeWindow_queryTransformHint(const ANativeWindow *psNativeWindow,
                                       int *piValue)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_query(psNativeWindow,
	                           ANATIVEWINDOW_QUERY_TRANSFORM_HINT,
	                           piValue);
#else
	return psNativeWindow->query(psNativeWindow,
	                             NATIVE_WINDOW_TRANSFORM_HINT,
	                             piValue);
#endif
}

static inline int
AndroidNativeWindow_getFormat(ANativeWindow *psNativeWindow,
                              int *piValue)
{
#if (API_LEVEL >= 26)
	*piValue = ANativeWindow_getFormat(psNativeWindow);
	return 0;
#else
	return psNativeWindow->query(psNativeWindow, NATIVE_WINDOW_FORMAT, piValue);
#endif
}

static inline int
AndroidNativeWindow_setBufferCount(ANativeWindow *psNativeWindow,
                                   size_t iBufferCnt)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setBufferCount(psNativeWindow, iBufferCnt);
#else
	return native_window_set_buffer_count(psNativeWindow, iBufferCnt);
#endif
}

static inline int
AndroidNativeWindow_setBuffersDimensions(ANativeWindow *psNativeWindow,
                                         int iWidth, int iHeight)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setBuffersDimensions(psNativeWindow, (int32_t)iWidth,
	                                          (uint32_t)iHeight);
#else
	return native_window_set_buffers_dimensions(psNativeWindow, iWidth,
	                                            iHeight);
#endif
}

static inline int
AndroidNativeWindow_setBuffersFormat(ANativeWindow *psNativeWindow, int iFormat)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setBuffersFormat(psNativeWindow, iFormat);
#else
	return native_window_set_buffers_format(psNativeWindow, iFormat);
#endif
}

static inline int
AndroidNativeWindow_setBuffersTransform(ANativeWindow *psNativeWindow,
                                        int iTransform)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setBuffersTransform(psNativeWindow, iTransform);
#else
	return native_window_set_buffers_transform(psNativeWindow, iTransform);
#endif
}

static inline int
AndroidNativeWindow_setSharedBufferMode(ANativeWindow *psNativeWindow,
                                        bool bEnabled)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setUsage(psNativeWindow, bEnabled);
#else
	return native_window_set_shared_buffer_mode(psNativeWindow, bEnabled);
#endif
}

static inline int
AndroidNativeWindow_setSwapInterval(ANativeWindow *psNativeWindow,
                                    int iInterval)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setSwapInterval(psNativeWindow, iInterval);
#else
	return psNativeWindow->setSwapInterval(psNativeWindow, iInterval);
#endif
}

static inline int
AndroidNativeWindow_setUsage(ANativeWindow *psNativeWindow, uint64_t ui64Usage)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_setUsage(psNativeWindow, ui64Usage);
#else
	return native_window_set_usage(psNativeWindow, ui64Usage);
#endif
}

static inline int
AndroidNativeWindow_cancelBuffer(ANativeWindow *psNativeWindow,
                                 ANativeWindowBuffer *psNativeBuffer,
                                 int iFenceFd)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_cancelBuffer(psNativeWindow, psNativeBuffer,
                                      iFenceFd);
#else
	return psNativeWindow->cancelBuffer(psNativeWindow, psNativeBuffer,
                                        iFenceFd);
#endif
}

static inline int
AndroidNativeWindow_queueBuffer(ANativeWindow *psNativeWindow,
                                ANativeWindowBuffer *psNativeBuffer,
                                int iFenceFd)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_queueBuffer(psNativeWindow, psNativeBuffer, iFenceFd);
#else
	return psNativeWindow->queueBuffer(psNativeWindow, psNativeBuffer,
	                                   iFenceFd);
#endif
}

static inline int
AndroidNativeWindow_dequeueBuffer(ANativeWindow *psNativeWindow,
                                  ANativeWindowBuffer **ppsNativeBuffer,
                                  int *piFenceFd)
{
#if (API_LEVEL >= 26)
	return ANativeWindow_dequeueBuffer(psNativeWindow, ppsNativeBuffer,
	                                   piFenceFd);
#else
	return psNativeWindow->dequeueBuffer(psNativeWindow, ppsNativeBuffer,
	                                     piFenceFd);
#endif
}

#if defined(PVR_ANDROID_HAS_GRALLOC_1)
#include <errno.h>

#include "gralloc_ddk_api.h"

static inline int usage_consumer_priv(int usage)
{
    return usage & GRALLOC_USAGE_PRIVATE_VALID_MASK;
}

static inline int usage_producer_priv(int usage)
{
    return (usage >> 16) & GRALLOC_USAGE_PRIVATE_VALID_MASK;
}

/* The compression hint is set into bits GRALLOC1_PRODUCER_USAGE_PRIVATE_{19
 * ->15} in MSB. GRALLOC1_PRODUCER_USAGE_PRIVATE_15 indicates if compression
 * hint is set already that '1' means a compression hint has already applied.
 * The buffer should use the compression type for rendering for the next frame.
 */
static inline int
AndroidNativeWindow_setCompression(struct ANativeWindow *psWindow,
                                   ANativeWindowBuffer *psWindowBuffer,
                                   uint32_t ui32Compression)
{
	uint64_t ui64ProducerPrivUage = 0, ui64ConsumerPrivUage = 0, ui64PrivUsage;
	uint64_t ui64Usage = 0;

	/* Mask out invalid compression types. */
	ui32Compression &= 0xF;

	if (psWindowBuffer)
	{
		IMG_native_handle_t *psNativeHandle =
			(IMG_native_handle_t *)psWindowBuffer->handle;

		ui64Usage = psNativeHandle->usage;

		ui64ProducerPrivUage = usage_producer_priv(psNativeHandle->iPrivUsage);
		ui64ConsumerPrivUage = usage_consumer_priv(psNativeHandle->iPrivUsage);

		/* Clear any existing compression bits */
		ui64ProducerPrivUage &= 0xFFE0;
	}

	/* Enable compression hint. */
	ui64ProducerPrivUage |= (uint64_t)GRALLOC_USAGE_COMPRESSION_HINT;
	/* Set compression hint. */
	ui64ProducerPrivUage |= (uint64_t)ui32Compression;

	ui64PrivUsage = ui64ProducerPrivUage | ui64ConsumerPrivUage;
	ui64Usage |= (ui64PrivUsage << GRALLOC_USAGE_PRIVATE_SHIFT);

	return AndroidNativeWindow_setUsage(psWindow, ui64Usage);
}
#endif /* PVR_ANDROID_HAS_GRALLOC_1 */

#endif /* IMG_NATIVEWINDOW_H */
