/*************************************************************************/ /*!
@File
@Title          Services dma transfer interface
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    Client side interface for dma operations scheduling
@License        Strictly Confidential.
*/ /**************************************************************************/

#ifndef PVRSRV_DMA_H
#define PVRSRV_DMA_H

#include "services.h"
#include "pvrsrv_devmem.h"
#include "pvrsrv_devmem_miw.h"
#include "pvrsrv_devmemx.h"
#include "pvrsrv_sync_um.h"
#include "pvrsrv_devmem_miw.h"

typedef struct DMA_OP_TAG {
	void *pvCPUAddr;			/* Pointer to the CPU buffer */
	PVRSRV_MEMINFO* psDevMemInfo;		/* Mem info of device memory buffer */
	IMG_DEVMEM_OFFSET_T uiOffset;		/* Offset into the device buffer */
	IMG_DEVMEM_SIZE_T uiSize;		/* Size of the transfer */
} DMA_OP;

#define PVRSRV_DMA_FLAG_MEM_TO_DEV  (1U<<0)
#define PVRSRV_DMA_FLAG_DEV_TO_MEM  (0U<<0)

#define PVRSRV_DMA_FLAG_SYNCHRONOUS (1U<<1)

/* Opaque dma context handle used with dma api */
typedef struct PVRSRV_DMA_TRANSFER_CONTEXT_TAG *PVRSRV_DMA_TRANSFER_CONTEXT;

/*************************************************************************/ /*!
@Function       PVRSRVCreateDmaTransferContext
@Description    Creates a context object for dma transfer api.

@Input          psDevConnection     Device connection
@Input          phCtx               Pointer to a handle of context object to
                                    be filled by the function.

@Return         Standard PVRSRV_ERROR error code.
*/ /**************************************************************************/
IMG_EXPORT PVRSRV_ERROR
PVRSRVCreateDmaTransferContext(PVRSRV_DEV_CONNECTION *psDevConnection,
							   PVRSRV_DMA_TRANSFER_CONTEXT *phCtx);


/*************************************************************************/ /*!
@Function       PVRSRVDestroyDmaTransferContext
@Description    Destroys a dma transfer api context object.

@Input          psDevConnection     Device connection
@Input          hCtx                Handle to a context object to be destroyed.

@Return         Standard PVRSRV_ERROR error code.
*/ /**************************************************************************/
IMG_EXPORT PVRSRV_ERROR
PVRSRVDestroyDmaTransferContext(PVRSRV_DEV_CONNECTION	*psDevConnection,
								PVRSRV_DMA_TRANSFER_CONTEXT	hCtx);

//#define MAX_DMA_OPS (36U)
/*************************************************************************/ /*!
@Function       PVRSRVDmaTransfer
@Description    Execute an array of transfer operations

@Input          psDevConnection         Device connection
@Input          hCtx                    Handle to dma transfer context object
@Input          uiFlags                 Flags PVRSRV_DMA_FLAG_* describing
                                        paramteres of the transfer
@Input          uiNumDMAOps             Number of dma operations to schedule
                                        (size of psDMAOps array)
@Input          psDMAOps                Array containing uiNumDMA DMA_OP descriptors
@Input          hCheckFence             Input fence
@Input          phUpdateFence           Output fence
@Input          pcszUpdateFenceName     Output fence name

@Return         Standard PVRSRV_ERROR error code.
*/ /**************************************************************************/
IMG_EXPORT PVRSRV_ERROR
PVRSRVDmaTransfer(
		PVRSRV_DEV_CONNECTION *psDevConnection,
		PVRSRV_DMA_TRANSFER_CONTEXT hCtx,
		IMG_UINT32 uiFlags,
		IMG_UINT32 uiNumDMAOps,
		DMA_OP* psDMAOps,
		PVRSRV_FENCE hCheckFence,
		PVRSRV_FENCE* phUpdateFence,
		const char* pcszUpdateFenceName);

/*************************************************************************/ /*!
@Function       PVRSRVDmaDeviceParams
@Description    Gather DMA hardware restriction parameters

@Input          hCtx                Handle to dma transfer context object
@Output         pui32DmaAlign       DMA device transfer buffer alignment
@Output         pui32DmaSegment     DMA device transfer quantum

@Return         Standard PVRSRV_ERROR error code.
*/ /**************************************************************************/
IMG_EXPORT PVRSRV_ERROR
PVRSRVDmaDeviceParams(PVRSRV_DMA_TRANSFER_CONTEXT hCtx,
					  IMG_UINT32 *pui32DmaAlign,
					  IMG_UINT32 *pui32DmaSegment);
#endif /* PVRSRV_DMA_H */
