/*************************************************************************/ /*!
@File
@Title          Device Memory Management allocation flags for internal Services
                use only
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    This file defines flags used on memory allocations and mappings
                These flags are relevant throughout the memory management
                software stack and are specified by users of services and
                understood by all levels of the memory management in the server
                and in special cases in the client.
@License        Strictly Confidential.
*/ /**************************************************************************/

#ifndef PVRSRV_MEMALLOCFLAGS_INTERNAL_H
#define PVRSRV_MEMALLOCFLAGS_INTERNAL_H

/*!
   CPU domain. Request uncached memory. This means that any writes to memory
   allocated with this flag are written straight to memory and thus are
   coherent for any device in the system.
*/
#define PVRSRV_MEMALLOCFLAG_CPU_UNCACHED (1ULL<<11)

/*!
  @Description    Macro checking whether the PVRSRV_MEMALLOCFLAG_CPU_UNCACHED mode is set.
  @Input  uiFlags Allocation flags.
  @Return         True if the mode is set, false otherwise
 */
#define PVRSRV_CHECK_CPU_UNCACHED(uiFlags) (PVRSRV_CPU_CACHE_MODE(uiFlags) == PVRSRV_MEMALLOCFLAG_CPU_UNCACHED)

/*!
 * Memory will be uncached on CPU and GPU
 */
#define PVRSRV_MEMALLOCFLAG_UNCACHED (PVRSRV_MEMALLOCFLAG_GPU_UNCACHED | PVRSRV_MEMALLOCFLAG_CPU_UNCACHED)

/*!
  @Description    Macro checking whether the PVRSRV_MEMALLOCFLAG_UNCACHED mode is set.
  @Input  uiFlags Allocation flags.
  @Return         True if the mode is set, false otherwise
 */
#define PVRSRV_CHECK_UNCACHED(uiFlags) (PVRSRV_CACHE_MODE(uiFlags) == PVRSRV_MEMALLOCFLAG_UNCACHED)

#endif /* PVRSRV_MEMALLOCFLAGS_INTERNAL_H */
