/*************************************************************************/ /*!
@File
@Title          RGX Common Types and Defines Header
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    Common types and definitions for RGX software
@License        Strictly Confidential.
*/ /**************************************************************************/
#ifndef RGX_COMMON_ASSERTS_H
#define RGX_COMMON_ASSERTS_H

#if defined(__cplusplus)
extern "C" {
#endif

/*! This macro represents a mask of LSBs that must be zero on data structure
 * sizes and offsets to ensure they are 8-byte granular on types shared between
 * the FW and host driver */
#define RGX_FW_ALIGNMENT_LSB (7U)

/*! Macro to test structure size alignment */
#define RGX_FW_STRUCT_SIZE_ASSERT(_a)	\
	static_assert((sizeof(_a) & RGX_FW_ALIGNMENT_LSB) == 0U,	\
				  "Size of " #_a " is not properly aligned")

/*! Macro to test structure member alignment */
#define RGX_FW_STRUCT_OFFSET_ASSERT(_a, _b)	\
	static_assert((offsetof(_a, _b) & RGX_FW_ALIGNMENT_LSB) == 0U,	\
				  "Offset of " #_a "." #_b " is not properly aligned")

#if defined(__cplusplus)
}
#endif

#endif /* RGX_COMMON_ASSERTS_H */

/******************************************************************************
 End of file
******************************************************************************/
