/*************************************************************************/ /*!
@File
@Title          RGX firmware signature checks
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    RGX firmware interface structures used by srvinit and server
@License        Strictly Confidential.
*/ /**************************************************************************/

#if !defined (RGX_FWIF_SIG_H)
#define RGX_FWIF_SIG_H

#include "rgxdefs_km.h"

/************************************************************************
* RGX FW signature checks
************************************************************************/

/* List of TA signature and checksum register addresses */
static const RGXFW_REGISTER_LIST asTASigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
#if defined(RGX_FEATURE_SCALABLE_VDM_GPP)
	{RGX_CR_USC_UVB_CHECKSUM,			RGX_CR_BLACKPEARL_INDIRECT,	0, RGX_NUM_PHANTOMS-1},
#else
	{RGX_CR_USC_UVS0_CHECKSUM,			0,							0, 0},
#if (RGX_FEATURE_NUM_CLUSTERS > 1)
	{RGX_CR_USC_UVS1_CHECKSUM,			0,							0, 0},
#if (RGX_FEATURE_NUM_CLUSTERS > 2)
	{RGX_CR_USC_UVS2_CHECKSUM,			0,							0, 0},
	{RGX_CR_USC_UVS3_CHECKSUM,			0,							0, 0},
#if (RGX_FEATURE_NUM_CLUSTERS > 4)
	{RGX_CR_USC_UVS4_CHECKSUM,			0,							0, 0},
	{RGX_CR_USC_UVS5_CHECKSUM,			0,							0, 0},
#endif
#endif
#endif
#endif
#if defined(RGX_FEATURE_SCALABLE_TE_ARCH)
#if defined(RGX_FEATURE_SCALABLE_VDM_GPP)
	{RGX_CR_PPP_CLIP_CHECKSUM,			RGX_CR_BLACKPEARL_INDIRECT,	0, RGX_NUM_PHANTOMS-1},
#else
	{RGX_CR_PPP,						0,							0, 0},
#endif
	{RGX_CR_TE_CHECKSUM,				0,							0, 0},
#else
	{RGX_CR_PPP_SIGNATURE,				0,							0, 0},
	{RGX_CR_TE_SIGNATURE,				0,							0, 0},
#endif
	{RGX_CR_VCE_CHECKSUM,				0,							0, 0},
#if !defined(RGX_FEATURE_PDS_PER_DUST) || !defined(FIX_HW_BRN_62204)
	{RGX_CR_PDS_DOUTM_STM_SIGNATURE,	0,							0, 0},
#endif
};

#if !defined(RGX_CR_PBE_INDIRECT)
#define HWR_SIG_PBE_INDIRECT (0)
#define HWR_SIG_PBE_INDIRECT_NUM (0)
#else
#define HWR_SIG_PBE_INDIRECT (RGX_CR_PBE_INDIRECT)
#define HWR_SIG_PBE_INDIRECT_NUM (RGX_FEATURE_NUM_CLUSTERS-1U)
#endif

#if defined(RGX_FEATURE_S7_TOP_INFRASTRUCTURE)
#define HWR_SIG_RAST_INDIRECT (RGX_CR_BLACKPEARL_INDIRECT)
#define HWR_SIG_RAST_INDIRECT_NUM (RGX_NUM_PHANTOMS-1U)
#else
#if defined(RGX_CR_RASTERISATION_INDIRECT)
#define HWR_SIG_RAST_INDIRECT (RGX_CR_RASTERISATION_INDIRECT)
#define HWR_SIG_RAST_INDIRECT_NUM (RGX_NUM_RASTERISATION_MODULES-1U)
#else
#define HWR_SIG_RAST_INDIRECT (0)
#define HWR_SIG_RAST_INDIRECT_NUM (0)
#endif
#endif /* defined(RGX_FEATURE_S7_TOP_INFRASTRUCTURE) */

/* List of 3D signature and checksum register addresses */
static const RGXFW_REGISTER_LIST as3DSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_ISP_PDS_CHECKSUM,           HWR_SIG_RAST_INDIRECT,  0, HWR_SIG_RAST_INDIRECT_NUM},
	{RGX_CR_ISP_TPF_CHECKSUM,           HWR_SIG_RAST_INDIRECT,  0, HWR_SIG_RAST_INDIRECT_NUM},
	{RGX_CR_TFPU_PLANE0_CHECKSUM,       HWR_SIG_RAST_INDIRECT,  0, HWR_SIG_RAST_INDIRECT_NUM},
	{RGX_CR_TFPU_PLANE1_CHECKSUM,       HWR_SIG_RAST_INDIRECT,  0, HWR_SIG_RAST_INDIRECT_NUM},
	{RGX_CR_PBE_CHECKSUM,               HWR_SIG_PBE_INDIRECT,   0, HWR_SIG_PBE_INDIRECT_NUM},
	{RGX_CR_IFPU_ISP_CHECKSUM,          HWR_SIG_RAST_INDIRECT,  0, HWR_SIG_RAST_INDIRECT_NUM},
};

#if defined (RGX_FEATURE_S7_TOP_INFRASTRUCTURE) || defined (RGX_FEATURE_PBE2_IN_XE)
/* Register present in (indirect) pbe regbank on:
 * - Rogue XT+
 * - Select Rogue XE cores with pbe2 feature */
#define SIG_PBE_CHECKSUM_INDIRECT     (RGX_CR_PBE_INDIRECT)
#define SIG_PBE_CHECKSUM_INDIRECT_NUM (RGX_FEATURE_NUM_CLUSTERS - 1U)
#else
/* Rogue XT: Tornado (direct)
 * Rogue (S6): bifpmcache (direct) */
#define SIG_PBE_CHECKSUM_INDIRECT     (0)
#define SIG_PBE_CHECKSUM_INDIRECT_NUM (0)
#endif

/* List of TDM checksum registers. All known registers are present only on select cores
 * wih TDM_PDS_CHECKSUM feature, hence wrapping this list under feature define */
 #if defined(RGX_FEATURE_TDM_PDS_CHECKSUM) || defined(RGX_FEATURE_PBE_CHECKSUM_2D)
static const RGXFW_REGISTER_LIST asTDMSigRegList[] =
{	/* Register */                    /* Indirect_Reg */               /* Start, End */
#if defined(RGX_FEATURE_TDM_PDS_CHECKSUM)
	{RGX_CR_TDM_PDS_CHECKSUM,         0 /* direct (present in HUB) */, 0, 0},
#endif
#if defined(RGX_FEATURE_PBE_CHECKSUM_2D)
	{RGX_CR_PBE_CHECKSUM_TDM,         SIG_PBE_CHECKSUM_INDIRECT,       0, SIG_PBE_CHECKSUM_INDIRECT_NUM},
#endif
};
#endif

#if defined(SUPPORT_VALIDATION) && defined(RGX_FEATURE_SCRATCH_REGISTERS)
#endif /* SUPPORT_VALIDATION */

#endif /*  RGX_FWIF_SIG_H */

/******************************************************************************
 End of file (rgx_fwif_sig.h)
******************************************************************************/
