/*************************************************************************/ /*!
@File
@Title          RGX firmware signature checks
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    RGX firmware interface structures used by srvinit and server
@License        Strictly Confidential.
*/ /**************************************************************************/

#if !defined (RGX_FWIF_SIG_H)
#define RGX_FWIF_SIG_H


/************************************************************************
* RGX FW signature checks
************************************************************************/



/* List of TDM signature and checksum register addresses */
static const RGXFW_REGISTER_LIST asTDMSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
#if defined(RGX_FEATURE_TDM_PDS_CHECKSUM)
	{RGX_CR_TDM_PDS_CHECKSUM,			0,							0, 0}, /* direct (present in jones) */
#endif
#if defined(RGX_FEATURE_PBE_CHECKSUM_2D)
	{RGX_CR_PBE_CHECKSUM_2D,			RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#else
	{RGX_CR_PBE_CHECKSUM,				RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#endif
#if (RGX_FEATURE_BIF_PER_PBE>RGX_FEATURE_PBE_PER_SPU)
	{RGX_CR_PBE_CHECKSUM_2D_BIF1,		RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#endif
};


/* List of TA signature and checksum register addresses */
static const RGXFW_REGISTER_LIST asTASigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_USC_UVB_CHECKSUM,			RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_PPP_CLIP_CHECKSUM,			RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_TE_CHECKSUM,				0,							0, 0},
	{RGX_CR_TE_TMA_CHECKSUM,			0,							0, 0},
	{RGX_CR_VCE_CHECKSUM,				RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_VCE_PRIM_CHECKSUM,			RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_PDS_DOUTM_STM_CHECKSUM,		RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#if defined(SUPPORT_RAY_TRACING)
	{RGX_CR_URI_RLOAD_CHECKSUM_RAY,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_URI_RSTORE_CHECKSUM_RAY,	RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_PRIMARY_CHECKSUM,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_SECONDARY_CHECKSUM,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#endif
};


/* List of 3D signature and checksum register addresses */
static const RGXFW_REGISTER_LIST as3DSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_ISP_PDS_CHECKSUM,			RGX_CR_ISP_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_NUM_ISP_PER_SPU)-1U},
	{RGX_CR_ISP_TPF_CHECKSUM,			RGX_CR_ISP_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_NUM_ISP_PER_SPU)-1U},
	{RGX_CR_TFPU_CHECKSUM,				RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#if defined(RGX_FEATURE_PBE_CHECKSUM_2D)
	{RGX_CR_PBE_CHECKSUM_3D,			RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#else
	{RGX_CR_PBE_CHECKSUM,				RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#endif
	{RGX_CR_IFPU_ISP_CHECKSUM,			RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#if defined(RGX_FEATURE_ZLS_CHECKSUM)
	{RGX_CR_ZLS_CHECKSUM,				RGX_CR_ISP_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_NUM_ISP_PER_SPU)-1U},
#endif
#if (RGX_FEATURE_BIF_PER_PBE>RGX_FEATURE_PBE_PER_SPU)
	{RGX_CR_PBE_CHECKSUM_3D_BIF1,		RGX_CR_PBE_INDIRECT,		0, (RGX_FEATURE_NUM_SPU*RGX_FEATURE_PBE_PER_SPU)-1U},
#endif
#if defined(SUPPORT_RAY_TRACING)
	{RGX_CR_URI_RLOAD_CHECKSUM_RAY,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_URI_RSTORE_CHECKSUM_RAY,	RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_PRIMARY_CHECKSUM,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_SECONDARY_CHECKSUM,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#endif
};


/* List of CDM signature and checksum register addresses */
static const RGXFW_REGISTER_LIST asCDMSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_CDM_PDS_CHECKSUM,			RGX_CR_TEXAS_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#if defined(SUPPORT_RAY_TRACING)
	{RGX_CR_URI_RLOAD_CHECKSUM_RAY,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_URI_RSTORE_CHECKSUM_RAY,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_PRIMARY_CHECKSUM,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_SECONDARY_CHECKSUM,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
#endif
};

#if defined(SUPPORT_RAY_TRACING)
/* List of RDM signature and checksum register addresses */
static const RGXFW_REGISTER_LIST asRDMSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_RCE_CHECKSUM,					RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_URI_RLOAD_CHECKSUM_RAY,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_URI_RSTORE_CHECKSUM_RAY,		RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_PRIMARY_CHECKSUM,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
	{RGX_CR_RTS_SECONDARY_CHECKSUM,			RGX_CR_RAC_INDIRECT,		0, RGX_FEATURE_NUM_SPU-1U},
};
#endif

#if defined(SUPPORT_VALIDATION)
static const RGXFW_REGISTER_LIST asValidationSigRegList[] =
{	/* Register */						/* Indirect_Reg */			/* Start, End */
	{RGX_CR_FBCDC_STATUS,				0,							0, 0},
	{RGX_CR_FBCDC_SIGNATURE_STATUS,		0,							0, 0},
};

#endif

#endif /*  RGX_FWIF_SIG_H */

/******************************************************************************
 End of file (rgx_fwif_sig.h)
******************************************************************************/


